# @Coding: UTF-8
# @Time: 2024/9/24 12:58
# @Author: xieyang_ls
# @Filename: annotation.py
from abc import ABC
from argparse import ArgumentTypeError

from http.server import BaseHTTPRequestHandler


def Get(path: str):
    if not isinstance(path, str):
        raise ValueError('GET Method: path should be a string')
    if len(path) == 0:
        raise ValueError('GET Method: path should not be empty')

    def decorator_get_func(func):
        func.__decorator__ = "GET"
        func.__decorator_path__ = path
        return func

    return decorator_get_func


def Post(path: str):
    if not isinstance(path, str):
        raise ValueError('POST Method: path should be a string')
    if len(path) == 0:
        raise ValueError('POST Method: path should not be empty')

    def decorator_post_func(func):
        func.__decorator__ = "POST"
        func.__decorator_path__ = path
        return func

    return decorator_post_func


def Put(path: str):
    if not isinstance(path, str):
        raise ValueError('PUT Method: path should be a string')
    if len(path) == 0:
        raise ValueError('PUT Method: path should not be empty')

    def decorator_put_func(func):
        func.__decorator__ = "PUT"
        func.__decorator_path__ = path
        return func

    return decorator_put_func


def Delete(path: str):
    if not isinstance(path, str):
        raise ValueError('DELETE Method: path should be a string')
    if len(path) == 0:
        raise ValueError('DELETE Method: path should not be empty')

    def decorator_delete_func(func):
        func.__decorator__ = "DELETE"
        func.__decorator_path__ = path
        return func

    return decorator_delete_func


def RequestInterceptor(interceptor_paths: set[str]) -> callable:
    if not isinstance(interceptor_paths, set):
        raise TypeError('RequestInterceptor: interceptor_paths must be a set')
    for interceptor_path in interceptor_paths:
        if not isinstance(interceptor_path, str):
            raise TypeError('RequestInterceptor: interceptor_paths must be a string set')

    def decorator_request_func(other_cls) -> type:
        if not isinstance(other_cls, type):
            raise TypeError('RequestInterceptor can only be applied to classes')
        print("RequestInterceptor called")
        other_cls.__decorator__ = "RequestInterceptor"
        other_cls.__decorator_params__ = interceptor_paths
        return other_cls

    return decorator_request_func


def InterceptorBefore() -> callable:
    def decorator_request_func(func) -> callable:

        def interceptor_before(request: BaseHTTPRequestHandler) -> tuple[int, bool]:
            if not isinstance(request, BaseHTTPRequestHandler):
                raise ArgumentTypeError('InterceptorBefore: argument must be BaseHTTPRequestHandler Type')
            request.headers["X-Intercepted"] = "True"
            response_code, response_status = func(request)
            if not isinstance(response_code, int) or not isinstance(response_status, bool):
                err: str = 'InterceptorBefore: return value must be (response_code: int, response_status: bool)'
                raise ValueError(err)
            return response_code, response_status

        func.__decorator__ = "InterceptorBefore"
        return interceptor_before

    return decorator_request_func


def InterceptorAfter() -> callable:
    def decorator_request_func(func) -> callable:
        def interceptor_after(request: BaseHTTPRequestHandler) -> None:
            if not isinstance(request, BaseHTTPRequestHandler):
                raise ArgumentTypeError('InterceptorAfter: argument should be BaseHTTPRequestHandler Type')
            func(request)
            return None

        func.__decorator__ = "InterceptorAfter"
        return interceptor_after

    return decorator_request_func



# @Coding: UTF-8
# @Time: 2024/9/14 21:47
# @Author: xieyang_ls
# @Filename: __init__.py.py

from pyutils_spirit.exception._exception import (ArgumentException,
                                                 NoneSignatureError,
                                                 ConflictSignatureError,
                                                 InvalidThreadWaitError,
                                                 SpiritContainerServiceException,
                                                 BusinessException,
                                                 SystemException)

__all__ = ['ArgumentException',
           'ConflictSignatureError',
           'NoneSignatureError',
           'InvalidThreadWaitError',
           'SpiritContainerServiceException',
           'BusinessException',
           'SystemException']



# @Coding: UTF-8
# @Time: 2024/9/11 21:54
# @Author: xieyang_ls
# @Filename: interpreter.py

from threading import Lock

from pymysql import Connect

from logging import info, basicConfig, INFO

from pyutils_spirit.util import Assemble, HashAssemble

from pyutils_spirit.exception import NoneSignatureError

basicConfig(level=INFO)


class Annotation:
    __connection = None

    __cursor = None

    __assemble: Assemble[str, object] = None

    __lock: Lock = None

    @classmethod
    def connection(cls, host: str, port: int, username: str, password: str, database: str) -> callable:
        def decorator_func(func) -> callable:
            def wrapper(*args, **kwargs):
                if cls.__connection is None or cls.__cursor is None:
                    cls.__connection = Connect(host=host, port=port, user=username, password=password)
                    cls.__connection.select_db(database)
                    cls.__cursor = cls.__connection.cursor()
                    info(f"Connected to database {database} is successfully")
                    func(*args, **kwargs)

            wrapper.__decorator__ = "connection"
            wrapper.__decorator_params = {"host": host, "port": port,
                                          "username": username,
                                          "password": password,
                                          "database": database}
            return wrapper

        return decorator_func

    @classmethod
    def singleton(cls, signature: str) -> callable:
        if cls.__lock is None:
            cls.__lock = Lock()
        cls.__lock.acquire()

        try:
            if not isinstance(signature, str):
                raise NoneSignatureError

            if cls.__assemble is None:
                cls.__assemble = HashAssemble()

            def get_signature(other_cls) -> callable:

                def get_instance(*args, **kwargs) -> object:
                    instance = cls.__assemble.get(signature)
                    if instance is None:
                        # 创建新实例并存储
                        instance = other_cls(*args, **kwargs)
                        cls.__assemble.put(signature, instance)
                    return instance

                get_instance.__decorator__ = "singleton"
                get_instance.__decorator_params__ = signature
                return get_instance

            return get_signature

        finally:
            cls.__lock.release()

    @classmethod
    def get_instance_signature(cls, signature: str) -> object:
        return cls.__assemble.get(signature)


connection = Annotation.connection
singleton = Annotation.singleton
get_instance_signature = Annotation.get_instance_signature



# @Coding: UTF-8
# @Time: 2024/9/21 22:56
# @Author: xieyang_ls
# @Filename: __init__.py.py

from pyutils_spirit.style.color import BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE, RESET

from pyutils_spirit.style.resources import (set_spirit_banner,
                                            set_websocket_banner,
                                            draw_spirit_banner,
                                            draw_websocket_banner)

__all__ = ["BLACK",
           "RED",
           "GREEN",
           "YELLOW",
           "BLUE",
           "MAGENTA",
           "CYAN",
           "WHITE",
           "RESET",
           "set_spirit_banner",
           "set_websocket_banner",
           "draw_spirit_banner",
           "draw_websocket_banner"]
